/**
*
* @module Kiwi
* @submodule Plugins
* @namespace Kiwi.Plugins
* @class GestureControls
*/
Kiwi.Plugins.GestureControls = {

	/**
	* The name of this plugin.
	* @property name
	* @type String
	* @default "GestureControls"
	* @public
	*/
	name: "GestureControls",

	/**
	* The version of this plugin.
	* @property version
	* @type String
	* @public
	*/
	version: "0.1.0",

	/**
	* Minimum version of Kiwi this plugin requires.
	* @property minimumKiwiVersion
	* @type String
	* @default '1.3.0'
	* @public
	*/
	minimumKiwiVersion: "1.3.0"

};

/**
* Registers this plugin with the Global Kiwi Plugins Manager if it is avaiable.
*/
Kiwi.PluginManager.register( Kiwi.Plugins.GestureControls );

/**
* Executed when the game is initially created. Assigns the game an internal reference to this object
* @method create
* @param game {Kiwi.Game} 
* @public
*/
Kiwi.Plugins.GestureControls.create = function( game ) {
    	
}


Kiwi.Plugins.GestureControls.Component = function( owner, params ) {

	params = params || {};

	Kiwi.Component.call( this, owner, 'GestureControls' );

	if( typeof params.enabled === "undefined" || params.enabled ) {
		this.enabled = true;
	} else {
		this.enabled = false;
	}


	this.pointers = [];
	this._transforms = [];
	this._initTransforms = [];

	if( typeof params.transforms !== "undefined" ) {

		if( Kiwi.Utils.Common.isArray( params.transforms) ) {
			this._transforms = params.transforms;
		} else {
			this._transforms = [params.transforms];
		}

	} else {
		this._transforms.push( this.owner.transform );
	}


	this.startAngle = 0;
	this.startDistance = 0;
	this.startX = 0;
	this.startY = 0;

	this.clone();

	this.game.input.onUp.add( this.onInputRelease, this, params.inputPriority );
	this.game.input.onDown.add( this.onInputPress, this, params.inputPriority );

};

Kiwi.extend( Kiwi.Plugins.GestureControls.Component, Kiwi.Component );


Kiwi.Plugins.GestureControls.Component.prototype.onInputRelease = function(x,y,td,tu,dur,pointer) {

	if( !this.enabled ) return;

	var i = this.pointers.length;
	var removed = false;

	while( i-- ) {
		if( this.pointers[i].id === pointer.id || !this.pointers[i].active) {
			this.pointers.splice(i, 1);
			removed = true;
		}
	}

	if( removed ) {
		this.clone();
	}
};


Kiwi.Plugins.GestureControls.Component.prototype.onInputPress = function(x,y,td,tu,dur,pointer) {

	if( !this.enabled || this.pointers.length >= 2 ) {
		return;
	} 

	var i = this.pointers.length;
	while(i--) {
		if( this.pointers[i].id === pointer.id ) {
			return;
		}
	}

	this.pointers.push( pointer );

	this.clone();

};

Kiwi.Plugins.GestureControls.Component.prototype.clone = function() {

	if( this.pointers.length >= 2 ) {
		
		var pOne = this.pointers[0];
		var pTwo = this.pointers[1];

		this.startAngle = pOne.point.angleTo( pTwo.point );
		this.startDistance = pOne.point.distanceTo( pTwo.point );

	} else if( this.pointers.length >= 1) {
		this.startX = this.pointers[0].point.x;
		this.startY = this.pointers[0].point.y;
	}

	this._initTransforms = [];

	for( var i = 0; i < this._transforms.length; i++) {
		this._initTransforms.push( this._transforms[i].clone() );
	}

};

Kiwi.Plugins.GestureControls.Component.prototype.update = function() {

	Kiwi.Component.prototype.update.call( this );


	if( this.pointers.length >= 2 ) {

		var pOne = this.pointers[0], 
			pTwo = this.pointers[1],
			angle, scale, i, transform, iTransform, anchorPointX, anchorPointY;

		//Angle
		angle = pOne.point.angleTo( pTwo.point ) - this.startAngle;

		//Scale
		scale = (pOne.point.distanceTo( pTwo.point ) / this.startDistance) - 1;
		
		//Position

		for( i = 0; i < this._transforms.length; i++) {
			transform = this._transforms[i];
			iTransform = this._initTransforms[i];

			transform.scaleX = iTransform.scaleX + scale;
			transform.scaleY = iTransform.scaleY + scale;
			transform.rotation = iTransform.rotation + angle; 
		}

	} else if( this.pointers.length === 1) {

		var pOne = this.pointers[0], 
			i, transform, iTransform;

		var x = pOne.point.x - this.startX;
		var y = pOne.point.y - this.startY;

		for( i = 0; i < this._transforms.length; i++) { 

			transform = this._transforms[i];
			iTransform = this._initTransforms[i];

			transform.x = iTransform.x + x;
			transform.y = iTransform.y + y;	

		}

	}


};


Kiwi.Plugins.GestureControls.Component.prototype.destroy = function() {

	this.game.input.onUp.remove( this.onInputRelease, this );
	this.game.input.onDown.remove( this.onInputPress, this);

	Kiwi.Component.prototype.destroy.call( this );

};